# written 2006-2007 by Markus Ullmann
# License: GPL-2


"""core module of maintainer-helper"""


from PyQt4 import QtGui, QtCore
from maintainer_helper.backend import arches, domain_repo_interface, get_cpv
from maintainer_helper.gui.MainWindow_ui import Ui_MainWindow
from maintainer_helper.gui.default_actions import default_actions
from maintainer_helper.gui.revdep_builder import RevdepMainWidget

CONFIGDIR = "~/.maintainer-helper"
DATADIR = "/usr/share/maintainer-helper"


class MainWidget(QtGui.QMainWindow):
    """Main Application window"""

    def __init__(self):
        QtGui.QMainWindow.__init__(self)

        # basic qt init
        self.useriface = Ui_MainWindow()
        self.useriface.setupUi(self)
        self.default_actions = default_actions(self)

        # init backend stuff
        self.archlist = arches(DATADIR).read_config_archlist()
        self.domain_repo_interface = domain_repo_interface()

        # some gui settings we can't do from designer
        self.useriface.keywordBox.setColumnCount(len(self.archlist))
        self.useriface.keywordBox.setHorizontalHeaderLabels(self.archlist)
        self.useriface.packageTree.sortItems(0, QtCore.Qt.SortOrder(0))

        # populate domain list
        for domain in self.domain_repo_interface.get_domainlist():
            self.useriface.domainSelect.insertItem(0, domain)

        # all common signal/slot connections we need
        self.connect(self.useriface.actionUsed_Arches,
            QtCore.SIGNAL("toggled(bool)"), self.refresh_keywords_bool)
        self.connect(self.useriface.applyFilter,
            QtCore.SIGNAL("clicked(bool)"), self.apply_filter)
        self.connect(self.useriface.actionAbout,
            QtCore.SIGNAL("triggered(bool)"), self.about_clicked)
        self.connect(self.useriface.actionCreate_Revdep_DB,
            QtCore.SIGNAL("triggered(bool)"), self.builddb_clicked)
        self.connect(self.useriface.actionOpen_Revdep_DB,
            QtCore.SIGNAL("triggered(bool)"), self.opendb_clicked)
        self.connect(self.useriface.actionAbout_Qt,
            QtCore.SIGNAL("triggered(bool)"), self.about_qt_clicked)
        self.connect(self.useriface.packageSearch,
            QtCore.SIGNAL("returnPressed()"),self.filter_return_pressed)
        self.connect(self.useriface.doEdit,
            QtCore.SIGNAL("clicked(bool)"),self.default_actions.do_edit)
        self.connect(self.useriface.doVersionBump,
            QtCore.SIGNAL("clicked(bool)"),self.default_actions.do_version_bump)
        self.connect(self.useriface.doEditPmask,
            QtCore.SIGNAL("clicked(bool)"),self.default_actions.do_edit_pmask)
        self.connect(self.useriface.doEbuildBump,
            QtCore.SIGNAL("clicked(bool)"),self.default_actions.do_ebuild_bump)
        self.connect(self.useriface.doDrop,
            QtCore.SIGNAL("clicked(bool)"),self.default_actions.do_drop_version)
        self.connect(self.useriface.domainSelect,
            QtCore.SIGNAL("currentIndexChanged(int)"),
            self.domain_select_changed)

        # imitate a changed domain selection to init the rest
        if self.useriface.domainSelect.currentIndex() != 0:
            self.useriface.domainSelect.setCurrentIndex(0)
        else:
            self.domain_select_changed(0)

        self.has_revdep_db = False

    def domain_select_changed(self, index):
        """called when the Combobox selection changed

        @param index: index int of the new selection
        """

        # disconnect all following events for now
        #(would senseless trigger refreshes)
        self.disconnect(self.useriface.repoSelect,
            QtCore.SIGNAL("currentIndexChanged(int)"), self.repo_select_changed)

        selected_domain = str(self.useriface.domainSelect.currentText())
        self.domain_repo_interface.set_domain_by_name(selected_domain)

        if self.useriface.repoSelect.count() > 0:
            self.useriface.repoSelect.clear()

        for repo in self.domain_repo_interface.repo_id_list:
            self.useriface.repoSelect.insertItem(0, repo)

        self.useriface.repoSelect.setCurrentIndex(0)
        self.connect(self.useriface.repoSelect,
            QtCore.SIGNAL("currentIndexChanged(int)"), self.repo_select_changed)

        self.repo_select_changed(0)

    def repo_select_changed(self, index):
        """A new repository is selected
        called on changed(int) of repoSelect ComboBox

        @param index: index of new selection (unused atm)
        """

        selected_repo = str(self.useriface.repoSelect.currentText())
        self.domain_repo_interface.set_work_repo_by_id(selected_repo)

        if self.useriface.applyFilter.isChecked():
            self.refresh_package_tree(str(self.useriface.packageSearch.text()))
        else:
            self.refresh_package_tree()

        self.refresh_changelog()
        self.refresh_keywords()
        self.refresh_application_data()

    def refresh_package_tree(self, filtertext=None):
        """Update the packageTree window"""

        triggerlist = frozenset([self.refresh_keywords, self.refresh_application_data,
            self.refresh_changelog, self.refresh_revdep])

        category_dict = {}
        itemlist = []

        # disable unwanted events until we're done (speed is mandatory!)
        for function in triggerlist:
            self.disconnect(self.useriface.packageTree,
                QtCore.SIGNAL("itemSelectionChanged()"), function)
        self.useriface.packageTree.setSortingEnabled(False)

        while self.useriface.packageTree.topLevelItemCount() > 0:
            self.useriface.packageTree.takeTopLevelItem(0)

        for package in self.domain_repo_interface.all_packages_list:
            cat = package[0]
            pkg = package[1]
            if filtertext is not None:
                if pkg.find(filtertext) < 0:
                    continue
            if cat not in category_dict:
                itemlist.append(QtGui.QTreeWidgetItem(len(itemlist)))
                category_dict[cat] = itemlist[len(itemlist)-1]
                category_dict[cat].setText(0, cat)
                category_dict[cat].setToolTip(0, cat)
                self.useriface.packageTree.addTopLevelItem(category_dict[cat])

            itemlist.append(QtGui.QTreeWidgetItem(len(itemlist)))
            package_item = itemlist[len(itemlist)-1]
            package_item.setText(0, pkg)
            category_dict[cat].addChild(package_item)

        self.useriface.packageTree.setHeaderLabels(["packages", ])
        if filtertext is not None:
            self.useriface.packageTree.expandAll()
        else:
            self.useriface.packageTree.collapseAll()

        for function in triggerlist:
            self.connect(self.useriface.packageTree,
                QtCore.SIGNAL("itemSelectionChanged()"), function)
        self.useriface.packageTree.setSortingEnabled(True)

    def get_packagetree_entry(self):
        """build atom from selected entry in packageTree"""

        try:
            cur_item = self.useriface.packageTree.selectedItems()[0]
            cat_name = str(cur_item.parent().text(0))
        except AttributeError:
            return None
        except IndexError:
            return None
        return (cat_name.encode('ascii'), str(cur_item.text(0)).encode('ascii'))

    def refresh_changelog(self):
        """Put in ChangeLog if an app is selected"""

        packagetree_entry = self.get_packagetree_entry()

        if packagetree_entry is None:
            self.useriface.changeLogBox.clear()
            return

        self.useriface.changeLogBox.setText(
            self.domain_repo_interface.get_changelog(packagetree_entry))

        self.useriface.changeLogBox.moveCursor(QtGui.QTextCursor.Start)

    def revdep_msg(self, msg):
        """Write message into RevdepTable only"""
        self.useriface.revdepTable.clear()
        self.useriface.revdepTable.setColumnCount(1)
        self.useriface.revdepTable.setRowCount(1)
        newitem = QtGui.QTableWidgetItem(msg)
        newitem.setFlags(QtCore.Qt.ItemIsSelectable)
        self.useriface.revdepTable.setItem(0, 0, newitem)
        self.useriface.revdepTable.resizeColumnsToContents()
        return

    def refresh_revdep(self):
        """Fill Revdep Table if an app is selected"""

        revdepTable = self.useriface.revdepTable

        if not self.has_revdep_db:
            self.revdep_msg("Please open a Revdep DB")
            return

        packagetree_entry = self.get_packagetree_entry()

        if packagetree_entry is None:
            self.revdep_msg("No package selected")
            return

        revdepTable.clear()
        revdepTable.setColumnCount(2)
        revdepTable.setHorizontalHeaderLabels(["Depend Reason", "Package depending"])
        hintitem =  revdepTable.horizontalHeaderItem(0)
        hintitem.setToolTip("NOTE: In DB-Only Mode, only keys are shown,\n not the actual reason")

        for row in self.revdep_db.execute(
            "SELECT rdependlist FROM rdepends WHERE atom='%s/%s'" % packagetree_entry):
            shown_atoms = 0
            for atom in sorted([get_cpv(rdep) for rdep in row[0].split(", ") if len(rdep)]):
                revdepTable.setRowCount(shown_atoms + 1)
                leftitem = QtGui.QTableWidgetItem("%s/%s" % packagetree_entry)
                leftitem.setToolTip("NOTE: In DB-Only Mode, only keys are shown,\n"
                    " not the actual reason")
                leftitem.setFlags(QtCore.Qt.ItemIsSelectable)
                rightitem = QtGui.QTableWidgetItem(atom.cpvstr)
                rightitem.setFlags(QtCore.Qt.ItemIsSelectable|QtCore.Qt.ItemIsEnabled)
                revdepTable.setItem(shown_atoms, 0, leftitem)
                revdepTable.setItem(shown_atoms, 1, rightitem)
                shown_atoms += 1
            if not shown_atoms:
                self.revdep_msg("No reverse dependencies for this package")
            revdepTable.resizeColumnsToContents()

    def refresh_application_data(self):
        """refresh metadata boxes"""

        packagetree_entry = self.get_packagetree_entry()

        if packagetree_entry is None:
            self.useriface.descriptionLabel.setText('')
            self.useriface.homepageLabel.setText('')
            self.useriface.herdsLabel.setText('')
            self.useriface.maintainerLabel.setText('')
            return

        app_metadata = self.domain_repo_interface.get_app_metadata(
            packagetree_entry)
        (description, homepage, herds, maintainers) = app_metadata
        self.useriface.descriptionLabel.setText(unicode(description))
        self.useriface.homepageLabel.setText(
            "<a href=\"" + homepage + "\">" + homepage + "</a>")
        self.useriface.herdsLabel.setText(herds)
        self.useriface.maintainerLabel.setText(maintainers)

    def refresh_keywords_bool(self, boolvalue):
        """helper function if the caller gives a bool"""

        if self.useriface.keywordBox.rowCount() > 0:
            self.refresh_keywords()

    def refresh_keywords(self):
        """refresh keyword box"""

        used_arches = []
        self.useriface.keywordBox.setRowCount(0)

        packagetree_entry = self.get_packagetree_entry()

        if packagetree_entry is None:
            self.refresh_selected_ebuild(-1, -1)
            self.disconnect(self.useriface.keywordBox,
                QtCore.SIGNAL("cellClicked(int,int)"),
                self.refresh_selected_ebuild)
            return

        self.connect(self.useriface.keywordBox,
            QtCore.SIGNAL("cellClicked(int,int)"),
            self.refresh_selected_ebuild)

        for lineno, (version, keyword_dict) in enumerate(
            self.domain_repo_interface.get_package_keywords_dict(
                packagetree_entry)):
            self.useriface.keywordBox.setRowCount(lineno + 1)
            for column, arch in enumerate(self.archlist):
                newitem = QtGui.QTableWidgetItem()
                if arch in keyword_dict:
                    if arch not in used_arches:
                        used_arches.append(arch)
                    if keyword_dict[arch].startswith("M"):
                        newitem.setBackground(QtGui.QBrush(
                            QtGui.QColor("orangered")))
                    elif keyword_dict[arch] == ("~"):
                        newitem.setBackground(QtGui.QBrush(
                            QtGui.QColor("yellow")))
                    else:
                        newitem.setBackground(QtGui.QBrush(
                            QtGui.QColor("limegreen")))
                    newitem.setText(keyword_dict[arch])
                newitem.setFlags(QtCore.Qt.ItemIsSelectable |
                    QtCore.Qt.ItemIsEnabled)
                newitem.setTextAlignment(QtCore.Qt.AlignHCenter |
                    QtCore.Qt.AlignVCenter)
                self.useriface.keywordBox.setItem(lineno, column, newitem)

            newitem = QtGui.QTableWidgetItem()
            newitem.setToolTip(version)
            newitem.setText(version)
            self.useriface.keywordBox.setVerticalHeaderItem(lineno, newitem)

        if self.useriface.actionUsed_Arches.isChecked():
            for arch in self.archlist:
                if arch not in used_arches:
                    self.useriface.keywordBox.setColumnHidden(
                        self.archlist.index(arch), 1)
                else:
                    self.useriface.keywordBox.setColumnHidden(
                        self.archlist.index(arch), 0)
        else:
            for colno in xrange(0, len(self.archlist)):
                self.useriface.keywordBox.setColumnHidden(colno, 0)

        self.useriface.keywordBox.resizeRowsToContents()
        self.useriface.keywordBox.resizeColumnsToContents()
        self.useriface.keywordBox.repaint()

        self.statusBar().clearMessage()

    def pkgtree_hide_items(self, text):
        """hide some packages that match a text

        @param text: text to grep for
        """

        self.statusBar().showMessage("Filtering package list")
        self.repaint()

        if len(text) > 0:
            self.refresh_package_tree(text)
        else:
            self.refresh_package_tree()

        self.statusBar().clearMessage()

    def hide_empty_groups_refresh(self, hideit):
        """refresh for hide_empty_groups"""

        self.pkgtree_hide_items(self.useriface.packageSearch.text())

    def refresh_selected_ebuild(self, row, column):
        """user selected a version to work on"""

        if row < 0 or column < 0:
            self.useriface.selectedEbuildLabel.setText("select an ebuild")
            self.useriface.doEdit.setEnabled(False)
            #self.useriface.doKeyword.setEnabled(False)
            self.useriface.doEbuildBump.setEnabled(False)
            self.useriface.doDrop.setEnabled(False)
            self.useriface.removeFiles.setEnabled(False)
            return

        self.useriface.doEdit.setEnabled(True)
        #self.useriface.doKeyword.setEnabled(True)
        self.useriface.doEbuildBump.setEnabled(True)
        self.useriface.doDrop.setEnabled(True)
        self.useriface.removeFiles.setEnabled(True)

        (category, package) = self.get_packagetree_entry()
        version = str(self.useriface.keywordBox.verticalHeaderItem(row).text())
        self.useriface.selectedEbuildLabel.setText("%s/%s-%s" % (
            category, package, version))

    def get_current_version(self):
        """return the current version string"""

        row = self.useriface.keywordBox.currentRow()

        return str(self.useriface.keywordBox.verticalHeaderItem(row).text())

    def apply_filter(self, apply_bool):
        """bool filter for searching packageTree

        @param apply_bool: True/False for applying it
        """

        if apply_bool:
            self.connect(self.useriface.packageSearch,
                QtCore.SIGNAL("textChanged(QString)"), self.pkgtree_hide_items)
            self.connect(self.useriface.actionHide_empty_groups,
                QtCore.SIGNAL("toggled(bool)"),self.hide_empty_groups_refresh)
            self.pkgtree_hide_items(self.useriface.packageSearch.text())
        else:
            self.disconnect(self.useriface.packageSearch,
                QtCore.SIGNAL("textChanged(QString)"), self.pkgtree_hide_items)
            self.disconnect(self.useriface.actionHide_empty_groups,
                QtCore.SIGNAL("toggled(bool)"),self.hide_empty_groups_refresh)
            self.pkgtree_hide_items("")

    def filter_return_pressed(self):
        """When return is hit in the textbox, enable filters"""

        if not self.useriface.applyFilter.isChecked():
            self.useriface.applyFilter.toggle()
        self.apply_filter(True)

    def builddb_clicked(self, toggle):
        """call gui for revdep building"""

        RevdepMainWidget(self)

    def opendb_clicked(self, toggle):
        """open an existing revdep db"""

        dbname = str(QtGui.QFileDialog.getOpenFileName(self,
            "Open Revdep DB File", "", "Revdep DBs (*.pdb)"))
        if not dbname:
            return
        try:
            import sqlite3 as sqlite
        except ImportError:
            try:
                import pysqlite2 as sqlite
            except ImportError:
                QtGui.QMessageBox.critical(self, "No sqlite support", "Your system has no sqlite support")
                return
        self.revdep_db = sqlite.connect(dbname)
        self.has_revdep_db = True
        self.refresh_revdep()

    def about_clicked(self, toggle):
        """The almighty about box"""

        QtGui.QMessageBox.about(self, "About maintainerHelper",
"""maintainerHelper written 2006-2007 by Markus Ullmann <jokey@gentoo.org>

Icon(s) are used from CrystalSVG KDE Theme.

Using pkgcore and portage APIs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; version 2.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
""")

    def about_qt_clicked(self, toggle):
        """We are proud of Qt"""

        QtGui.QMessageBox.aboutQt(self, "About Qt")

